@@ -296,13 +296,7 @@ task_to_str (int taskno, const ada_task_info *task_info)
 static struct ada_tasks_pspace_data *
 get_ada_tasks_pspace_data (struct program_space *pspace)
 {
-  struct ada_tasks_pspace_data *data;
-
-  data = ada_tasks_pspace_data_handle.get (pspace);
-  if (data == NULL)
-    data = ada_tasks_pspace_data_handle.emplace (pspace);
-
-  return data;
+  return &ada_tasks_pspace_data_handle.try_emplace (pspace);
 }
 
 /* Return the ada-tasks module's data for the given inferior (INF).
@@ -320,13 +314,7 @@ get_ada_tasks_pspace_data (struct program_space *pspace)
 static struct ada_tasks_inferior_data *
 get_ada_tasks_inferior_data (struct inferior *inf)
 {
-  struct ada_tasks_inferior_data *data;
-
-  data = ada_tasks_inferior_data_handle.get (inf);
-  if (data == NULL)
-    data = ada_tasks_inferior_data_handle.emplace (inf);
-
-  return data;
+  return &ada_tasks_inferior_data_handle.try_emplace (inf);
 }
 
 /* Return the task number of the task whose thread is THREAD, or zero
@@ -477,15 +465,15 @@ read_fat_string_value (char *dest, struct value *val, int max_len)
 
   /* Get the size of the task image by checking the value of the bounds.
      The lower bound is always 1, so we only need to read the upper bound.  */
-  bounds_val = value_ind (value_field (val, bounds_fieldno));
-  len = value_as_long (value_field (bounds_val, upper_bound_fieldno));
+  bounds_val = value_ind (val->field (bounds_fieldno));
+  len = value_as_long (bounds_val->field (upper_bound_fieldno));
 
   /* Make sure that we do not read more than max_len characters...  */
   if (len > max_len)
     len = max_len;
 
   /* Extract LEN characters from the fat string.  */
-  array_val = value_ind (value_field (val, array_fieldno));
+  array_val = value_ind (val->field (array_fieldno));
   read_memory (array_val->address (), (gdb_byte *) dest, len);
 
   /* Add the NUL character to close the string.  */
@@ -640,13 +628,11 @@ ptid_from_atcb_common (struct value *common_value)
   const struct ada_tasks_pspace_data *pspace_data
     = get_ada_tasks_pspace_data (current_program_space);
 
-  ll_value = value_field (common_value, pspace_data->atcb_fieldno.ll);
+  ll_value = common_value->field (pspace_data->atcb_fieldno.ll);
 
   if (pspace_data->atcb_fieldno.ll_lwp >= 0)
-    lwp = value_as_address (value_field (ll_value,
-					 pspace_data->atcb_fieldno.ll_lwp));
-  thread = value_as_long (value_field (ll_value,
-				       pspace_data->atcb_fieldno.ll_thread));
+    lwp = value_as_address (ll_value->field (pspace_data->atcb_fieldno.ll_lwp));
+  thread = value_as_long (ll_value->field (pspace_data->atcb_fieldno.ll_thread));
 
   ptid = target_get_ada_task_ptid (lwp, thread);
 
@@ -684,7 +670,7 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
 
   tcb_value = value_from_contents_and_address (pspace_data->atcb_type,
 					       NULL, task_id);
-  common_value = value_field (tcb_value, pspace_data->atcb_fieldno.common);
+  common_value = tcb_value->field (pspace_data->atcb_fieldno.common);
 
   /* Fill in the task_id.  */
 
@@ -709,8 +695,7 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
     {
       if (pspace_data->atcb_fieldno.image >= 0)
 	read_fat_string_value (task_info->name,
-			       value_field (common_value,
-					    pspace_data->atcb_fieldno.image),
+			       common_value->field (pspace_data->atcb_fieldno.image),
 			       sizeof (task_info->name) - 1);
       else
 	{
@@ -741,31 +726,26 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
   else
     {
       int len = value_as_long
-		  (value_field (common_value,
-				pspace_data->atcb_fieldno.image_len));
+	(common_value->field (pspace_data->atcb_fieldno.image_len));
 
       value_as_string (task_info->name,
-		       value_field (common_value,
-				    pspace_data->atcb_fieldno.image),
+		       common_value->field (pspace_data->atcb_fieldno.image),
 		       len);
     }
 
   /* Compute the task state and priority.  */
 
   task_info->state =
-    value_as_long (value_field (common_value,
-				pspace_data->atcb_fieldno.state));
+    value_as_long (common_value->field (pspace_data->atcb_fieldno.state));
   task_info->priority =
-    value_as_long (value_field (common_value,
-				pspace_data->atcb_fieldno.priority));
+    value_as_long (common_value->field (pspace_data->atcb_fieldno.priority));
 
   /* If the ATCB contains some information about the parent task,
      then compute it as well.  Otherwise, zero.  */
 
   if (pspace_data->atcb_fieldno.parent >= 0)
     task_info->parent =
-      value_as_address (value_field (common_value,
-				     pspace_data->atcb_fieldno.parent));
+      value_as_address (common_value->field (pspace_data->atcb_fieldno.parent));
 
   /* If the task is in an entry call waiting for another task,
      then determine which task it is.  */
@@ -778,10 +758,10 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
 	 entry of another task; then the Task_Id of the called task is
 	 in My_ATCB.Entry_Calls (My_ATCB.ATC_Nesting_Level).Called_Task.  */
       atc_nesting_level_value =
-	value_field (tcb_value, pspace_data->atcb_fieldno.atc_nesting_level);
+	tcb_value->field (pspace_data->atcb_fieldno.atc_nesting_level);
       entry_calls_value =
 	ada_coerce_to_simple_array_ptr
-	  (value_field (tcb_value, pspace_data->atcb_fieldno.entry_calls));
+	(tcb_value->field (pspace_data->atcb_fieldno.entry_calls));
       entry_calls_value_element =
 	value_subscript (entry_calls_value,
 			 value_as_long (atc_nesting_level_value));
@@ -789,8 +769,7 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
 	ada_get_field_index (entry_calls_value_element->type (),
 			     "called_task", 0);
       task_info->called_task =
-	value_as_address (value_field (entry_calls_value_element,
-				       called_task_fieldno));
+	value_as_address (entry_calls_value_element->field (called_task_fieldno));
     }
 
   /* If the ATCB contains some information about RV callers, then
@@ -801,8 +780,7 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
       /* Get the ID of the caller task from Common_ATCB.Call.all.Self.
 	 If Common_ATCB.Call is null, then there is no caller.  */
       const CORE_ADDR call =
-	value_as_address (value_field (common_value,
-				       pspace_data->atcb_fieldno.call));
+	value_as_address (common_value->field (pspace_data->atcb_fieldno.call));
       struct value *call_val;
 
       if (call != 0)
@@ -812,14 +790,13 @@ read_atcb (CORE_ADDR task_id, struct ada_task_info *task_info)
 					     NULL, call);
 	  task_info->caller_task =
 	    value_as_address
-	      (value_field (call_val, pspace_data->atcb_fieldno.call_self));
+	    (call_val->field (pspace_data->atcb_fieldno.call_self));
 	}
     }
 
   task_info->base_cpu
     = (pspace_data->cpu_id_offset
-      + value_as_long (value_field (common_value,
-				    pspace_data->atcb_fieldno.base_cpu)));
+       + value_as_long (common_value->field (pspace_data->atcb_fieldno.base_cpu)));
 
   /* And finally, compute the task ptid.  Note that there is not point
      in computing it if the task is no longer alive, in which case
@@ -899,10 +876,9 @@ read_known_tasks_list (struct ada_tasks_inferior_data *data)
       /* Read the chain.  */
       tcb_value = value_from_contents_and_address (pspace_data->atcb_type,
 						   NULL, task_id);
-      common_value = value_field (tcb_value, pspace_data->atcb_fieldno.common);
+      common_value = tcb_value->field (pspace_data->atcb_fieldno.common);
       task_id = value_as_address
-		  (value_field (common_value,
-				pspace_data->atcb_fieldno.activation_link));
+	(common_value->field (pspace_data->atcb_fieldno.activation_link));
     }
 
   return true;
