@@ -1,6 +1,6 @@
 /* Python frame unwinder interface.
 
-   Copyright (C) 2015-2025 Free Software Foundation, Inc.
+   Copyright (C) 2015-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -66,10 +66,8 @@ show_pyuw_debug (struct ui_file *file, int from_tty,
       }							     \
   } while (0)
 
-struct pending_frame_object
+struct pending_frame_object : public PyObject
 {
-  PyObject_HEAD
-
   /* Frame we are unwinding.  */
   frame_info_ptr frame_info;
 
@@ -94,10 +92,8 @@ struct saved_reg
 /* The data we keep for the PyUnwindInfo: pending_frame, saved registers
    and frame ID.  */
 
-struct unwind_info_object
+struct unwind_info_object : public PyObject
 {
-  PyObject_HEAD
-
   /* gdb.PendingFrame for the frame we are unwinding.  */
   PyObject *pending_frame;
 
@@ -484,7 +480,7 @@ pending_framepy_read_register (PyObject *self, PyObject *args, PyObject *kw)
   if (!gdbpy_parse_register_id (pending_frame->gdbarch, pyo_reg_id, &regnum))
     return nullptr;
 
-  PyObject *result = nullptr;
+  gdbpy_ref<> result;
   try
     {
       scoped_value_mark free_values;
@@ -508,7 +504,7 @@ pending_framepy_read_register (PyObject *self, PyObject *args, PyObject *kw)
       return gdbpy_handle_gdb_exception (nullptr, except);
     }
 
-  return result;
+  return result.release ();
 }
 
 /* Implement PendingFrame.is_valid().  Return True if this pending frame
@@ -603,8 +599,6 @@ pending_framepy_language (PyObject *self, PyObject *args)
     {
       return gdbpy_handle_gdb_exception (nullptr, except);
     }
-
-  Py_RETURN_NONE;
 }
 
 /* Implement PendingFrame.find_sal().  Return the PendingFrame's symtab and
@@ -617,21 +611,17 @@ pending_framepy_find_sal (PyObject *self, PyObject *args)
 
   PENDING_FRAMEPY_REQUIRE_VALID (pending_frame);
 
-  PyObject *sal_obj = nullptr;
-
   try
     {
       frame_info_ptr frame = pending_frame->frame_info;
 
       symtab_and_line sal = find_frame_sal (frame);
-      sal_obj = symtab_and_line_to_sal_object (sal);
+      return symtab_and_line_to_sal_object (sal).release ();
     }
   catch (const gdb_exception &except)
     {
       return gdbpy_handle_gdb_exception (nullptr, except);
     }
-
-  return sal_obj;
 }
 
 /* Implement PendingFrame.block().  Return a gdb.Block for the pending
@@ -670,7 +660,8 @@ pending_framepy_block (PyObject *self, PyObject *args)
       return nullptr;
     }
 
-  return block_to_block_object (block, fn_block->function ()->objfile ());
+  return block_to_block_object (block,
+				fn_block->function ()->objfile ()).release ();
 }
 
 /* Implement gdb.PendingFrame.function().  Return a gdb.Symbol
@@ -700,7 +691,7 @@ pending_framepy_function (PyObject *self, PyObject *args)
     }
 
   if (sym != nullptr)
-    return symbol_to_symbol_object (sym);
+    return symbol_to_symbol_object (sym).release ();
 
   Py_RETURN_NONE;
 }
@@ -769,7 +760,7 @@ pending_framepy_architecture (PyObject *self, PyObject *args)
 
   PENDING_FRAMEPY_REQUIRE_VALID (pending_frame);
 
-  return gdbarch_to_arch_object (pending_frame->gdbarch);
+  return gdbarch_to_arch_object (pending_frame->gdbarch).release ();
 }
 
 /* Implementation of PendingFrame.level (self) -> Integer.  */
@@ -998,9 +989,7 @@ static const registry<gdbarch>::key<pyuw_gdbarch_data_type> pyuw_gdbarch_data;
 static void
 pyuw_on_new_gdbarch (gdbarch *newarch)
 {
-  struct pyuw_gdbarch_data_type *data = pyuw_gdbarch_data.get (newarch);
-  if (data == nullptr)
-    data= pyuw_gdbarch_data.emplace (newarch);
+  struct pyuw_gdbarch_data_type *data = &pyuw_gdbarch_data.try_emplace (newarch);
 
   if (!data->unwinder_registered)
     {
