@@ -1,6 +1,6 @@
 /* frv simulator fr400 dependent profiling code.
 
-   Copyright (C) 2001-2021 Free Software Foundation, Inc.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
    Contributed by Red Hat
 
 This file is part of the GNU simulators.
@@ -632,7 +632,6 @@ frvbf_model_fr400_u_media_1 (SIM_CPU *cpu, const IDESC *idesc,
 {
   int cycles;
   FRV_PROFILE_STATE *ps;
-  const CGEN_INSN *insn;
   int busy_adjustment[] = {0, 0};
   int *fr;
 
@@ -643,7 +642,6 @@ frvbf_model_fr400_u_media_1 (SIM_CPU *cpu, const IDESC *idesc,
   cycles = idesc->timing->units[unit_num].done;
 
   ps = CPU_PROFILE_STATE (cpu);
-  insn = idesc->idata;
 
   /* The latency of the registers may be less than previously recorded,
      depending on how they were used previously.
@@ -1673,9 +1671,7 @@ frvbf_model_fr400_u_media_4 (SIM_CPU *cpu, const IDESC *idesc,
 {
   int cycles;
   FRV_PROFILE_STATE *ps;
-  const CGEN_INSN *insn;
   int busy_adjustment[] = {0};
-  int *fr;
 
   if (model_insn == FRV_INSN_MODEL_PASS_1)
     return 0;
@@ -1684,7 +1680,6 @@ frvbf_model_fr400_u_media_4 (SIM_CPU *cpu, const IDESC *idesc,
   cycles = idesc->timing->units[unit_num].done;
 
   ps = CPU_PROFILE_STATE (cpu);
-  insn = idesc->idata;
 
   /* The latency of the registers may be less than previously recorded,
      depending on how they were used previously.
@@ -1709,7 +1704,6 @@ frvbf_model_fr400_u_media_4 (SIM_CPU *cpu, const IDESC *idesc,
   post_wait_for_FR (cpu, out_FRk);
 
   /* Restore the busy cycles of the registers we used.  */
-  fr = ps->fr_busy;
 
   /* The latency of the output register will be at least the latency of the
      other inputs.  Once initiated, post-processing will take 1 cycle.  */
@@ -1751,7 +1745,6 @@ frvbf_model_fr400_u_media_4_acc_dual (SIM_CPU *cpu, const IDESC *idesc,
 {
   int cycles;
   FRV_PROFILE_STATE *ps;
-  const CGEN_INSN *insn;
   INT ACC40Si_1;
   INT FRk_1;
 
@@ -1765,8 +1758,6 @@ frvbf_model_fr400_u_media_4_acc_dual (SIM_CPU *cpu, const IDESC *idesc,
   ACC40Si_1 = DUAL_REG (in_ACC40Si);
   FRk_1 = DUAL_REG (out_FRk);
 
-  insn = idesc->idata;
-
   /* The post processing must wait if there is a dependency on a FR
      which is not ready yet.  */
   ps->post_wait = cycles;
@@ -1802,7 +1793,6 @@ frvbf_model_fr400_u_media_6 (SIM_CPU *cpu, const IDESC *idesc,
 {
   int cycles;
   FRV_PROFILE_STATE *ps;
-  const CGEN_INSN *insn;
   int busy_adjustment[] = {0};
   int *fr;
 
@@ -1813,7 +1803,6 @@ frvbf_model_fr400_u_media_6 (SIM_CPU *cpu, const IDESC *idesc,
   cycles = idesc->timing->units[unit_num].done;
 
   ps = CPU_PROFILE_STATE (cpu);
-  insn = idesc->idata;
 
   /* The latency of the registers may be less than previously recorded,
      depending on how they were used previously.
